@@ -42,18 +42,18 @@
 #include <algorithm>
 #include <limits>
 
-#include "proj.h"
-#include "proj_experimental.h"
-#include "proj_internal.h"
+#include "R-libproj/proj.h"
+#include "R-libproj/proj_experimental.h"
+#include "R-libproj/proj_internal.h"
 #include <math.h>
-#include "geodesic.h"
-#include "grids.hpp"
-#include "filemanager.hpp"
+#include "R-libproj/geodesic.h"
+#include "R-libproj/grids.hpp"
+#include "R-libproj/filemanager.hpp"
 
-#include "proj/common.hpp"
-#include "proj/coordinateoperation.hpp"
-#include "proj/internal/internal.hpp"
-#include "proj/internal/io_internal.hpp"
+#include "R-libproj/proj/common.hpp"
+#include "R-libproj/proj/coordinateoperation.hpp"
+#include "R-libproj/proj/internal/internal.hpp"
+#include "R-libproj/proj/internal/io_internal.hpp"
 
 using namespace NS_PROJ::internal;
 
@@ -176,7 +176,8 @@ double proj_roundtrip (PJ *P, PJ_DIRECTION direction, int n, PJ_COORD *coord) {
         return HUGE_VAL;
 
     if (n < 1) {
-        proj_errno_set (P, EINVAL);
+        proj_log_error(P, _("n should be >= 1"));
+        proj_errno_set (P, PROJ_ERR_OTHER_API_MISUSE);
         return HUGE_VAL;
     }
 
@@ -202,7 +203,7 @@ double proj_roundtrip (PJ *P, PJ_DIRECTION direction, int n, PJ_COORD *coord) {
 
 /**************************************************************************************/
 int pj_get_suggested_operation(PJ_CONTEXT*,
-                               const std::vector<CoordOperation>& opList,
+                               const std::vector<PJCoordOperation>& opList,
                                const int iExcluded[2],
                                PJ_DIRECTION direction,
                                PJ_COORD coord)
@@ -242,7 +243,15 @@ int pj_get_suggested_operation(PJ_CONTEXT*,
             // onshore. So in a general way, prefer a onshore area to a
             // offshore one.
             if( iBest < 0 ||
-                (alt.accuracy >= 0 && alt.accuracy < bestAccuracy &&
+                (alt.accuracy >= 0 &&
+                (alt.accuracy < bestAccuracy ||
+                 // If two operations have the same accuracy, use the one that
+                 // is contained within a larger one
+                 (alt.accuracy == bestAccuracy &&
+                  alt.minxSrc > opList[iBest].minxSrc &&
+                  alt.minySrc > opList[iBest].minySrc &&
+                  alt.maxxSrc < opList[iBest].maxxSrc &&
+                  alt.maxySrc < opList[iBest].maxySrc)) &&
                 !alt.isOffshore) ) {
                 iBest = i;
                 bestAccuracy = alt.accuracy;
@@ -291,6 +300,15 @@ similarly, but prefers the 2D resp. 3D interfaces if available.
             if( iBest < 0 ) {
                 break;
             }
+            if( iRetry > 0 ) {
+                const int oldErrno = proj_errno_reset(P);
+                if (proj_log_level(P->ctx, PJ_LOG_TELL) >= PJ_LOG_DEBUG) {
+                    pj_log(P->ctx, PJ_LOG_DEBUG, proj_context_errno_string(P->ctx, oldErrno));
+                }
+                pj_log(P->ctx, PJ_LOG_DEBUG,
+                    "Did not result in valid result. "
+                    "Attempting a retry with another operation.");
+            }
 
             const auto& alt = P->alternativeCoordinateOperations[iBest];
             if( P->iCurCoordOp != iBest ) {
@@ -303,15 +321,12 @@ similarly, but prefers the 2D resp. 3D interfaces if available.
             }
             PJ_COORD res = direction == PJ_FWD ?
                         pj_fwd4d( coord, alt.pj ) : pj_inv4d( coord, alt.pj );
-            if( proj_errno(alt.pj) == PJD_ERR_NETWORK_ERROR ) {
+            if( proj_errno(alt.pj) == PROJ_ERR_OTHER_NETWORK_ERROR ) {
                 return proj_coord_error ();
             }
             if( res.xyzt.x != HUGE_VAL ) {
                 return res;
             }
-            pj_log(P->ctx, PJ_LOG_DEBUG,
-                   "Did not result in valid result. "
-                   "Attempting a retry with another operation.");
             if( iRetry == N_MAX_RETRY ) {
                 break;
             }
@@ -339,6 +354,8 @@ similarly, but prefers the 2D resp. 3D interfaces if available.
                         if (proj_log_level(P->ctx, PJ_LOG_TELL) >= PJ_LOG_DEBUG) {
                             std::string msg("Using coordinate operation ");
                             msg += alt.name;
+                            msg += " as a fallback due to lack of more "
+                                   "appropriate operations";
                             pj_log(P->ctx, PJ_LOG_DEBUG, msg.c_str());
                         }
                         P->iCurCoordOp = i;
@@ -353,21 +370,14 @@ similarly, but prefers the 2D resp. 3D interfaces if available.
             }
         }
 
-        proj_errno_set (P, EINVAL);
+        proj_errno_set (P, PROJ_ERR_COORD_TRANSFM_NO_OPERATION);
         return proj_coord_error ();
     }
 
-    switch (direction) {
-        case PJ_FWD:
-            return pj_fwd4d (coord, P);
-        case PJ_INV:
-            return  pj_inv4d (coord, P);
-        default:
-            break;
-    }
-
-    proj_errno_set (P, EINVAL);
-    return proj_coord_error ();
+    if (direction == PJ_FWD)
+        return pj_fwd4d (coord, P);
+    else
+        return pj_inv4d (coord, P);
 }
 
 
@@ -377,18 +387,43 @@ int proj_trans_array (PJ *P, PJ_DIRECTION direction, size_t n, PJ_COORD *coord)
 /******************************************************************************
     Batch transform an array of PJ_COORD.
 
+    Performs transformation on all points, even if errors occur on some points.
+
+    Individual points that fail to transform will have their components set to
+    HUGE_VAL
+
     Returns 0 if all coordinates are transformed without error, otherwise
-    returns error number.
+    returns a precise error number if all coordinates that fail to transform
+    for the same reason, or a generic error code if they fail for different
+    reasons.
 ******************************************************************************/
     size_t i;
+    int retErrno = 0;
+    bool hasSetRetErrno = false;
+    bool sameRetErrno = true;
 
     for (i = 0;  i < n;  i++) {
+        proj_context_errno_set(P->ctx, 0);
         coord[i] = proj_trans (P, direction, coord[i]);
-        if (proj_errno(P))
-            return proj_errno (P);
-    }
+        int thisErrno = proj_errno(P);
+        if( thisErrno != 0 )
+        {
+            if( !hasSetRetErrno )
+            {
+                retErrno = thisErrno;
+                hasSetRetErrno = true;
+            }
+            else if( sameRetErrno && retErrno != thisErrno )
+            {
+                sameRetErrno = false;
+                retErrno = PROJ_ERR_COORD_TRANSFM;
+            }
+        }
+   }
 
-   return 0;
+   proj_context_errno_set(P->ctx, retErrno);
+
+   return retErrno;
 }
 
 
@@ -494,9 +529,6 @@ size_t proj_trans_generic (
             break;
         case PJ_IDENT:
             return nmin;
-        default:
-            proj_errno_set (P, EINVAL);
-            return 0;
     }
 
     /* Arrays of length==0 are broadcast as the constant 0               */
@@ -765,22 +797,22 @@ PJ *pj_create_internal (PJ_CONTEXT *ctx, const char *definition) {
     n = strlen (definition);
     args = (char *) malloc (n + 1);
     if (nullptr==args) {
-        proj_context_errno_set(ctx, ENOMEM);
+        proj_context_errno_set(ctx, PROJ_ERR_OTHER /*ENOMEM*/);
         return nullptr;
     }
     strcpy (args, definition);
 
     argc = pj_trim_argc (args);
     if (argc==0) {
-        pj_dealloc (args);
-        proj_context_errno_set(ctx, PJD_ERR_NO_ARGS);
+        free (args);
+        proj_context_errno_set(ctx, PROJ_ERR_INVALID_OP_MISSING_ARG);
         return nullptr;
     }
 
     argv = pj_trim_argv (argc, args);
     if (!argv) {
-        pj_dealloc(args);
-        proj_context_errno_set(ctx, ENOMEM);
+        free(args);
+        proj_context_errno_set(ctx, PROJ_ERR_OTHER /*ENOMEM*/);
         return nullptr;
     }
 
@@ -789,8 +821,8 @@ PJ *pj_create_internal (PJ_CONTEXT *ctx, const char *definition) {
     allow_init_epsg = proj_context_get_use_proj4_init_rules(ctx, FALSE);
     P = pj_init_ctx_with_allow_init_epsg (ctx, (int) argc, argv, allow_init_epsg);
 
-    pj_dealloc (argv);
-    pj_dealloc (args);
+    free (argv);
+    free (args);
 
     /* Support cs2cs-style modifiers */
     ret = cs2cs_emulation_setup  (P);
@@ -815,20 +847,20 @@ indicator, as in {"+proj=utm", "+zone=32"}, or leave it out, as in {"proj=utm",
     if (nullptr==ctx)
         ctx = pj_get_default_ctx ();
     if (nullptr==argv) {
-        proj_context_errno_set(ctx, PJD_ERR_NO_ARGS);
+        proj_context_errno_set(ctx, PROJ_ERR_INVALID_OP_MISSING_ARG);
         return nullptr;
     }
 
     /* We assume that free format is used, and build a full proj_create compatible string */
     c = pj_make_args (argc, argv);
     if (nullptr==c) {
-        proj_context_errno_set(ctx, ENOMEM);
+        proj_context_errno_set(ctx, PROJ_ERR_INVALID_OP /* ENOMEM */);
         return nullptr;
     }
 
     P = proj_create (ctx, c);
 
-    pj_dealloc ((char *) c);
+    free ((char *) c);
     return P;
 }
 
@@ -843,26 +875,26 @@ Same as proj_create_argv() but calls pj_create_internal() instead of proj_create
     if (nullptr==ctx)
         ctx = pj_get_default_ctx ();
     if (nullptr==argv) {
-        proj_context_errno_set(ctx, PJD_ERR_NO_ARGS);
+        proj_context_errno_set(ctx, PROJ_ERR_INVALID_OP_MISSING_ARG);
         return nullptr;
     }
 
     /* We assume that free format is used, and build a full proj_create compatible string */
     c = pj_make_args (argc, argv);
     if (nullptr==c) {
-        proj_context_errno_set(ctx, ENOMEM);
+        proj_context_errno_set(ctx, PROJ_ERR_OTHER /*ENOMEM*/);
         return nullptr;
     }
 
     P = pj_create_internal (ctx, c);
 
-    pj_dealloc ((char *) c);
+    free ((char *) c);
     return P;
 }
 
 /** Create an area of use */
 PJ_AREA * proj_area_create(void) {
-    return static_cast<PJ_AREA*>(pj_calloc(1, sizeof(PJ_AREA)));
+    return static_cast<PJ_AREA*>(calloc(1, sizeof(PJ_AREA)));
 }
 
 /** Assign a bounding box to an area of use. */
@@ -880,7 +912,7 @@ void proj_area_set_bbox(PJ_AREA *area,
 
 /** Free an area of use */
 void proj_area_destroy(PJ_AREA* area) {
-    pj_dealloc(area);
+    free(area);
 }
 
 /************************************************************************/
@@ -971,27 +1003,31 @@ static void reproject_bbox(PJ* pjGeogToCrs,
         maxx = -maxx;
         maxy = -maxy;
 
-        std::vector<double> x, y;
-        x.resize(21 * 4);
-        y.resize(21 * 4);
-        for( int j = 0; j <= 20; j++ )
+        constexpr int N_STEPS = 20;
+        constexpr int N_STEPS_P1 = N_STEPS+1;
+        constexpr int XY_SIZE = N_STEPS_P1 * 4;
+        std::vector<double> x(XY_SIZE);
+        std::vector<double> y(XY_SIZE);
+        const double step_lon = (east_lon - west_lon) / N_STEPS;
+        const double step_lat = (north_lat - south_lat) / N_STEPS;
+        for( int j = 0; j <= N_STEPS; j++ )
         {
-            x[j] = west_lon + j * (east_lon - west_lon) / 20;
+            x[j] = west_lon + j * step_lon;
             y[j] = south_lat;
-            x[21+j] = west_lon + j * (east_lon - west_lon) / 20;
-            y[21+j] = north_lat;
-            x[21*2+j] = west_lon;
-            y[21*2+j] = south_lat + j * (north_lat - south_lat) / 20;
-            x[21*3+j] = east_lon;
-            y[21*3+j] = south_lat + j * (north_lat - south_lat) / 20;
+            x[N_STEPS_P1+j] = x[j];
+            y[N_STEPS_P1+j] = north_lat;
+            x[N_STEPS_P1*2+j] = west_lon;
+            y[N_STEPS_P1*2+j] = south_lat + j * step_lat;
+            x[N_STEPS_P1*3+j] = east_lon;
+            y[N_STEPS_P1*3+j] = y[N_STEPS_P1*2+j];
         }
         proj_trans_generic (
             pjGeogToCrs, PJ_FWD,
-                &x[0], sizeof(double), 21 * 4,
-                &y[0], sizeof(double), 21 * 4,
+                &x[0], sizeof(double), XY_SIZE,
+                &y[0], sizeof(double), XY_SIZE,
                 nullptr, 0, 0,
                 nullptr, 0, 0);
-        for( int j = 0; j < 21 * 4; j++ )
+        for( int j = 0; j < XY_SIZE; j++ )
         {
             if( x[j] != HUGE_VAL && y[j] != HUGE_VAL )
             {
@@ -1014,7 +1050,7 @@ static PJ* add_coord_op_to_list(
                             PJ* pjGeogToSrc,
                             PJ* pjGeogToDst,
                             bool isOffshore,
-                            std::vector<CoordOperation>& altCoordOps) {
+                            std::vector<PJCoordOperation>& altCoordOps) {
 /*****************************************************************************/
 
     double minxSrc;
@@ -1063,28 +1099,18 @@ static PJ* create_operation_to_geog_crs(PJ_CONTEXT* ctx, const PJ* crs) {
         geodetic_crs_type == PJ_TYPE_GEOGRAPHIC_3D_CRS )
     {
         auto datum = proj_crs_get_datum(ctx, geodetic_crs);
-        if( datum )
-        {
-            auto cs = proj_create_ellipsoidal_2D_cs(
-                ctx, PJ_ELLPS2D_LONGITUDE_LATITUDE, nullptr, 0);
-            auto ellps = proj_get_ellipsoid(ctx, datum);
-            proj_destroy(datum);
-            double semi_major_metre = 0;
-            double inv_flattening = 0;
-            proj_ellipsoid_get_parameters(ctx, ellps, &semi_major_metre,
-                                          nullptr, nullptr, &inv_flattening);
-            auto temp = proj_create_geographic_crs(
-                ctx, "unnamed crs", "unnamed datum",
-                proj_get_name(ellps),
-                semi_major_metre, inv_flattening,
-                "Reference prime meridian", 0, nullptr, 0,
-                cs);
-            proj_destroy(ellps);
-            proj_destroy(cs);
-            proj_destroy(geodetic_crs);
-            geodetic_crs = temp;
-            geodetic_crs_type = proj_get_type(geodetic_crs);
-        }
+        auto datum_ensemble = proj_crs_get_datum_ensemble(ctx, geodetic_crs);
+        auto cs = proj_create_ellipsoidal_2D_cs(
+            ctx, PJ_ELLPS2D_LONGITUDE_LATITUDE, nullptr, 0);
+        auto temp = proj_create_geographic_crs_from_datum(
+            ctx, "unnamed crs", datum ? datum : datum_ensemble,
+            cs);
+        proj_destroy(datum);
+        proj_destroy(datum_ensemble);
+        proj_destroy(cs);
+        proj_destroy(geodetic_crs);
+        geodetic_crs = temp;
+        geodetic_crs_type = proj_get_type(geodetic_crs);
     }
     if( geodetic_crs_type != PJ_TYPE_GEOGRAPHIC_2D_CRS )
     {
@@ -1183,7 +1209,7 @@ PJ  *proj_create_crs_to_crs (PJ_CONTEXT *ctx, const char *source_crs, const char
 
 
 /*****************************************************************************/
-std::vector<CoordOperation> pj_create_prepared_operations(PJ_CONTEXT *ctx,
+std::vector<PJCoordOperation> pj_create_prepared_operations(PJ_CONTEXT *ctx,
                                                      const PJ *source_crs,
                                                      const PJ *target_crs,
                                                      PJ_OBJ_LIST* op_list)
@@ -1208,7 +1234,7 @@ std::vector<CoordOperation> pj_create_prepared_operations(PJ_CONTEXT *ctx,
 
     try
     {
-        std::vector<CoordOperation> preparedOpList;
+        std::vector<PJCoordOperation> preparedOpList;
 
         // Iterate over source->target candidate transformations and reproject
         // their long-lat bounding box into the source CRS.
@@ -1227,7 +1253,7 @@ std::vector<CoordOperation> pj_create_prepared_operations(PJ_CONTEXT *ctx,
                         &west_lon, &south_lat, &east_lon, &north_lat, &areaName) )
             {
                 const bool isOffshore =
-                    areaName && strstr(areaName, "offshore");
+                    areaName && strstr(areaName, "- offshore");
                 if( west_lon <= east_lon )
                 {
                     op = add_coord_op_to_list(i, op,
@@ -1266,8 +1292,20 @@ std::vector<CoordOperation> pj_create_prepared_operations(PJ_CONTEXT *ctx,
     }
 }
 
+// ---------------------------------------------------------------------------
+
+//! @cond Doxygen_Suppress
+static const char *getOptionValue(const char *option,
+                                  const char *keyWithEqual) noexcept {
+    if (ci_starts_with(option, keyWithEqual)) {
+        return option + strlen(keyWithEqual);
+    }
+    return nullptr;
+}
+//! @endcond
+
 /*****************************************************************************/
-PJ  *proj_create_crs_to_crs_from_pj (PJ_CONTEXT *ctx, const PJ *source_crs, const PJ *target_crs, PJ_AREA *area, const char* const *) {
+PJ  *proj_create_crs_to_crs_from_pj (PJ_CONTEXT *ctx, const PJ *source_crs, const PJ *target_crs, PJ_AREA *area, const char* const * options) {
 /******************************************************************************
     Create a transformation pipeline between two known coordinate reference
     systems.
@@ -1279,11 +1317,46 @@ PJ  *proj_create_crs_to_crs_from_pj (PJ_CONTEXT *ctx, const PJ *source_crs, cons
         ctx = pj_get_default_ctx();
     }
 
-    auto operation_ctx = proj_create_operation_factory_context(ctx, nullptr);
+    const char* authority = nullptr;
+    double accuracy = -1;
+    bool allowBallparkTransformations = true;
+    for (auto iter = options; iter && iter[0]; ++iter) {
+        const char *value;
+        if ((value = getOptionValue(*iter, "AUTHORITY="))) {
+            authority = value;
+        } else if ((value = getOptionValue(*iter, "ACCURACY="))) {
+            accuracy = pj_atof(value);
+        } else if ((value = getOptionValue(*iter, "ALLOW_BALLPARK="))) {
+            if( ci_equal(value, "yes") )
+                allowBallparkTransformations = true;
+            else if( ci_equal(value, "no") )
+                allowBallparkTransformations = false;
+            else {
+                ctx->logger(ctx->logger_app_data, PJ_LOG_ERROR,
+                            "Invalid value for ALLOW_BALLPARK option.");
+                return nullptr;
+            }
+        } else {
+            std::string msg("Unknown option :");
+            msg += *iter;
+            ctx->logger(ctx->logger_app_data, PJ_LOG_ERROR, msg.c_str());
+            return nullptr;
+        }
+    }
+
+    auto operation_ctx = proj_create_operation_factory_context(ctx, authority);
     if( !operation_ctx ) {
         return nullptr;
     }
 
+    proj_operation_factory_context_set_allow_ballpark_transformations(
+        ctx, operation_ctx, allowBallparkTransformations);
+
+    if( accuracy >= 0 ) {
+        proj_operation_factory_context_set_desired_accuracy(ctx, operation_ctx,
+                                                            accuracy);
+    }
+
     if( area && area->bbox_set ) {
         proj_operation_factory_context_set_area_of_interest(
                                             ctx,
@@ -1348,6 +1421,7 @@ PJ  *proj_create_crs_to_crs_from_pj (PJ_CONTEXT *ctx, const PJ *source_crs, cons
 
     P->alternativeCoordinateOperations = std::move(preparedOpList);
     // The returned P is rather dummy
+    P->descr = "Set of coordinate operations";
     P->iso_obj = nullptr;
     P->fwd = nullptr;
     P->inv = nullptr;
@@ -1359,17 +1433,13 @@ PJ  *proj_create_crs_to_crs_from_pj (PJ_CONTEXT *ctx, const PJ *source_crs, cons
     return P;
 }
 
-PJ *proj_destroy (PJ *P) {
-    pj_free (P);
-    return nullptr;
-}
 
 /*****************************************************************************/
 int proj_errno (const PJ *P) {
 /******************************************************************************
     Read an error level from the context of a PJ.
 ******************************************************************************/
-    return pj_ctx_get_errno (pj_get_ctx ((PJ *) P));
+    return proj_context_errno (pj_get_ctx ((PJ *) P));
 }
 
 /*****************************************************************************/
@@ -1380,7 +1450,7 @@ int proj_context_errno (PJ_CONTEXT *ctx) {
 ******************************************************************************/
     if (nullptr==ctx)
         ctx = pj_get_default_ctx();
-    return pj_ctx_get_errno (ctx);
+    return ctx->last_errno;
 }
 
 /*****************************************************************************/
@@ -1395,6 +1465,7 @@ int proj_errno_set (const PJ *P, int err) {
     /* For P==0 err goes to the default context */
     proj_context_errno_set (pj_get_ctx ((PJ *) P), err);
     errno = err;
+
     return err;
 }
 
@@ -1445,16 +1516,15 @@ int proj_errno_reset (const PJ *P) {
     int last_errno;
     last_errno = proj_errno (P);
 
-    pj_ctx_set_errno (pj_get_ctx ((PJ *) P), 0);
+    proj_context_errno_set (pj_get_ctx ((PJ *) P), 0);
     errno = 0;
-    pj_errno = 0;
     return last_errno;
 }
 
 
-/* Create a new context */
+/* Create a new context based on the default context */
 PJ_CONTEXT *proj_context_create (void) {
-    return pj_ctx_alloc ();
+    return new (std::nothrow) pj_ctx(*pj_get_default_ctx());
 }
 
 
@@ -1466,7 +1536,7 @@ PJ_CONTEXT *proj_context_destroy (PJ_CONTEXT *ctx) {
     if (pj_get_default_ctx ()==ctx)
         return nullptr;
 
-    pj_ctx_free (ctx);
+    delete ctx;
     return nullptr;
 }
 
@@ -1505,15 +1575,15 @@ static char *path_append (char *buf, const char *app, size_t *buf_size) {
 
     /* "pj_realloc", so to speak */
     if (*buf_size < len) {
-        p = static_cast<char*>(pj_calloc (2 * len, sizeof (char)));
+        p = static_cast<char*>(calloc (2 * len, sizeof (char)));
         if (nullptr==p) {
-            pj_dealloc (buf);
+            free (buf);
             return nullptr;
         }
         *buf_size = 2 * len;
         if (buf != nullptr)
             strcpy (p, buf);
-        pj_dealloc (buf);
+        free (buf);
         buf = p;
     }
     assert(buf);
@@ -1555,7 +1625,7 @@ PJ_INFO proj_info (void) {
 
     /* build search path string */
     auto ctx = pj_get_default_ctx();
-    if (!ctx || ctx->search_paths.empty()) {
+    if (ctx->search_paths.empty()) {
         const auto searchpaths = pj_get_default_searchpaths(ctx);
         for( const auto& path: searchpaths ) {
             buf = path_append(buf, path.c_str(), &buf_size);
@@ -1566,11 +1636,11 @@ PJ_INFO proj_info (void) {
         }
     }
 
-    pj_dalloc(const_cast<char*>(info.searchpath));
+    free(const_cast<char*>(info.searchpath));
     info.searchpath = buf ? buf : empty;
 
-    info.paths = ctx ? ctx->c_compat_paths : nullptr;
-    info.path_count = ctx ? static_cast<int>(ctx->search_paths.size()) : 0;
+    info.paths = ctx->c_compat_paths;
+    info.path_count = static_cast<int>(ctx->search_paths.size());
 
     pj_release_lock ();
     return info;
@@ -1641,7 +1711,7 @@ PJ_PROJ_INFO proj_pj_info(PJ *P) {
         pjinfo.definition = empty;
     else
         pjinfo.definition = pj_shrink (def);
-    /* Make pj_free clean this up eventually */
+    /* Make proj_destroy clean this up eventually */
     P->def_full = def;
 
     pjinfo.has_inverse = pj_has_inverse(P);
@@ -1752,7 +1822,7 @@ PJ_INIT_INFO proj_init_info(const char *initname){
         if( strcmp(initname, "epsg") == 0 || strcmp(initname, "EPSG") == 0 ) {
             const char* val;
 
-            pj_ctx_set_errno( ctx, 0 );
+            proj_context_errno_set( ctx, 0 );
 
             strncpy (ininfo.name, initname, sizeof(ininfo.name) - 1);
             strcpy(ininfo.origin, "EPSG");
@@ -1770,7 +1840,7 @@ PJ_INIT_INFO proj_init_info(const char *initname){
         if( strcmp(initname, "IGNF") == 0 ) {
             const char* val;
 
-            pj_ctx_set_errno( ctx, 0 );
+            proj_context_errno_set( ctx, 0 );
 
             strncpy (ininfo.name, initname, sizeof(ininfo.name) - 1);
             strcpy(ininfo.origin, "IGNF");
@@ -1815,7 +1885,7 @@ PJ_INIT_INFO proj_init_info(const char *initname){
 
     for ( ; start; start = next) {
         next = start->next;
-        pj_dalloc(start);
+        free(start);
     }
 
    return ininfo;
