@@ -37,9 +37,9 @@
 #define DEFMODEL_HPP
 
 #ifdef PROJ_COMPILATION
-#include "proj/internal/include_nlohmann_json.hpp"
+#include "R-libproj/proj/internal/include_nlohmann_json.hpp"
 #else
-#include "nlohmann/json.hpp"
+#include "R-libproj/nlohmann/json.hpp"
 #endif
 
 #include <algorithm>
@@ -54,7 +54,7 @@
 #define DEFORMATON_MODEL_NAMESPACE DeformationModel
 #endif
 
-#include "defmodel_exceptions.hpp"
+#include "R-libproj/transformations/defmodel_exceptions.hpp"
 
 namespace DEFORMATON_MODEL_NAMESPACE {
 
@@ -66,9 +66,9 @@ using json = nlohmann::json;
 class SpatialExtent {
   public:
     /** Parse the provided object as an extent.
-    *
-    * @throws ParsingException
-    */
+     *
+     * @throws ParsingException
+     */
     static SpatialExtent parse(const json &j);
 
     double minx() const { return mMinx; }
@@ -130,9 +130,9 @@ class Epoch {
 class Component {
   public:
     /** Parse the provided object as a component.
-    *
-    * @throws ParsingException
-    */
+     *
+     * @throws ParsingException
+     */
     static Component parse(const json &j);
 
     /** Get a text description of the component. */
@@ -181,7 +181,7 @@ class Component {
     /** Get the spatial model. */
     const SpatialModel &spatialModel() const { return mSpatialModel; }
 
-    /** Generic type for a type functon */
+    /** Generic type for a type function */
     struct TimeFunction {
         std::string type{};
 
@@ -218,11 +218,11 @@ class Component {
     };
 
     struct PiecewiseTimeFunction : public TimeFunction {
-        /** One of "zero", "constant", and "linear", defines the behaviour of
+        /** One of "zero", "constant", and "linear", defines the behavior of
          * the function before the first defined epoch */
         std::string beforeFirst{};
 
-        /** One of "zero", "constant", and "linear", defines the behaviour of
+        /** One of "zero", "constant", and "linear", defines the behavior of
          * the function after the last defined epoch */
         std::string afterLast{};
 
@@ -369,13 +369,13 @@ class MasterFile {
      * models. Typically "EPSG:XXXX" */
     const std::string &definitionCRS() const { return mDefinitionCRS; }
 
-    /** Get the nominal reference epoch of the deformation model. Formated
+    /** Get the nominal reference epoch of the deformation model. Formatted
      * as a ISO-8601 date-time. This is not necessarily used to calculate
      * the deformation model - each component defines its own time function. */
     const std::string &referenceEpoch() const { return mReferenceEpoch; }
 
     /** Get the epoch at which the uncertainties of the deformation model
-     * are calculated. Formated as a ISO-8601 date-time. */
+     * are calculated. Formatted as a ISO-8601 date-time. */
     const std::string &uncertaintyReferenceEpoch() const {
         return mUncertaintyReferenceEpoch;
     }
@@ -630,10 +630,10 @@ class Evaluator {
 
 // ---------------------------------------------------------------------------
 
-} // namespace DeformationModel
+} // namespace DEFORMATON_MODEL_NAMESPACE
 
 // ---------------------------------------------------------------------------
 
-#include "defmodel_impl.hpp"
+#include "R-libproj/transformations/defmodel_impl.hpp"
 
 #endif // DEFMODEL_HPP
